@@ -7,8 +7,8 @@
 #' As such, each _n_-leaf tree can be uniquely identified by a non-negative
 #' integer _x_ < `NUnrooted(n)`.
 #'
-#' This integer can be converted by a tree by treating it as a mixed-base number,
-#' with bases 1, 3, 5, 7, ... (2_n_ - 5).
+#' This integer can be converted by a tree by treating it as a mixed-base
+#' number, with bases 1, 3, 5, 7, ..., (2&nbsp;_n_ - 5).
 #'
 #' Each digit of this mixed base number corresponds to a leaf, and determines
 #' the location on a growing tree to which that leaf should be added.
@@ -70,13 +70,13 @@
 #' If we chose option 0 for the next two additions, we could specify this tree
 #' with the mixed-base number 0021.  We can convert this into decimal:
 #'
-#' 0 &times; (1 &times; 3 &times; 5 &times; 9) +
+#' 0 * (1 * 3 * 5 * 9) +
 #'
-#' 0 &times; (1 &times; 3 &times; 5) +
+#' 0 * (1 * 3 * 5) +
 #'
-#' 3 &times; (1 &times; 3) +
+#' 3 * (1 * 3) +
 #'
-#' 1 &times; (1)
+#' 1 * (1)
 #'
 #' = 10
 #'
@@ -92,8 +92,6 @@
 #' @template tipLabelsParam
 #' @param \dots Additional parameters for consistency with S3 methods (unused).
 #'
-#' @return `as.phylo.numeric` returns a tree of class `phylo`.
-#'
 #' @examples
 #' tree <- as.phylo(10, nTip = 6)
 #' plot(tree)
@@ -106,15 +104,15 @@
 #' treeNumber <- as.TreeNumber("1234567890123", nTip = 14)
 #' tree <- as.phylo(treeNumber)
 #'
-#' @seealso [`TreeShape`]
+#' @seealso Describe the shape of a tree topology, independent of leaf labels:
+#' [`TreeShape()`]
 #' @family tree generation functions
-#' @exportClass TreeNumber
 #' @name TreeNumber
 #
 
 #' @rdname TreeNumber
 #'
-#' @return `as.TreeNumber` returns an object of class `TreeNumber`,
+#' @return `as.TreeNumber()` returns an object of class `TreeNumber`,
 #' which comprises a numeric vector, whose elements represent successive
 #' nine-digit chunks of the decimal integer corresponding to the tree topology
 #' (in big endian order).  The `TreeNumber` object has attributes
@@ -129,8 +127,8 @@ as.TreeNumber.phylo <- function (x, ...) {
   x <- root(x, 1, resolve.root = TRUE)
   edge <- x$edge
   nTip <- NTip(x)
-  edge <- PostorderEdges(edge)
-  structure(edge_to_num(edge[, 1], edge[, 2], nTip),
+  edge <- Postorder(edge)
+  structure(.Int64(edge_to_num(edge[, 1], edge[, 2], nTip)),
             nTip = nTip,
             tip.labels = TipLabels(x),
             class = 'TreeNumber')
@@ -139,20 +137,13 @@ as.TreeNumber.phylo <- function (x, ...) {
 #' @rdname TreeNumber
 #' @export
 as.TreeNumber.multiPhylo <- function (x, ...) {
-  vapply(x, as.TreeNumber.phylo, structure(0, nTip = 0,
-                                           tip.labels = '0',
-                                           class = 'TreeNumber'))
+  lapply(x, as.TreeNumber.phylo)
 }
 
 #' @rdname TreeNumber
 #' @export
 as.TreeNumber.character <- function (x, nTip, tipLabels = TipLabels(nTip), ...) {
-  len <- nchar(x)
-  ends <- rev(len - (seq_len((len / 9L) + 1L) - 1L) * 9L)
-  starts <- pmax(1L, ends - 8L)
-
-  # Return:
-  structure(as.integer(substring(x, starts, ends)),
+  structure(as.integer64(x),
             nTip = nTip,
             tip.labels = tipLabels,
             class = 'TreeNumber')
@@ -160,6 +151,7 @@ as.TreeNumber.character <- function (x, nTip, tipLabels = TipLabels(nTip), ...)
 
 #' @rdname TreeNumber
 #' @template MRS
+#' @return `as.phylo.numeric()` returns a tree of class `phylo`.
 #' @references Based on a concept by John Tromp, employed in Li _et al._ 1996.
 #'
 #' \insertRef{Li1996}{TreeTools}
@@ -179,25 +171,43 @@ as.phylo.numeric <- function (x, nTip = attr(x, 'nTip'),
     }
   }
   if (is.null(tipLabels)) tipLabels <- paste0('t', seq_len(nTip))
-  if (length(x) > 1) {
-    structure(lapply(x, as.phylo.numeric, nTip = nTip, tipLabels = tipLabels),
-              tip.label = tipLabels, class='multiPhylo')
+  if (nTip == 1) {
+    SingleTaxonTree(tipLabels)
   } else {
-    edge <- RenumberEdges(num_to_parent(x, nTip), seq_len(nTip + nTip - 2L))
-    structure(list(edge = do.call(cbind, edge),
-                   tip.label = tipLabels,
-                   Nnode = nTip - 1L),
-              order = 'preorder',
-              class = 'phylo')
+    if (length(x) > 1) {
+      structure(lapply(x, as.phylo.numeric, nTip = nTip, tipLabels = tipLabels),
+                tip.label = tipLabels, class = 'multiPhylo')
+    } else {
+      edge <- RenumberEdges(num_to_parent(.Int64.to.C(x), nTip),
+                            seq_len(nTip + nTip - 2L))
+      structure(list(edge = do.call(cbind, edge),
+                     tip.label = tipLabels,
+                     Nnode = nTip - 1L),
+                order = 'preorder',
+                class = 'phylo')
+    }
   }
 }
 
+#' @export
+as.phylo.integer64 <- as.phylo.numeric
+
+.Int64.to.C <- function (i64) {
+  INT_MAX <- 2147483647L
+  if (i64 > INT_MAX) c(i64 %/% INT_MAX, i64 %% INT_MAX) else as.integer(i64[1])
+}
+
+.TreeNumber.to.C <- function (tn64) {
+  .Int64.to.C(structure(tn64[1], class = 'integer64'))
+}
+
 #' @rdname TreeNumber
 #' @export
 as.phylo.TreeNumber <- function (x, nTip = attr(x, 'nTip'),
                                  tipLabels = attr(x, 'tip.label'), ...) {
   if (is.null(tipLabels)) tipLabels <- paste0('t', seq_len(nTip))
-  edge <- RenumberEdges(num_to_parent(x, nTip), seq_len(nTip + nTip - 2L))
+  edge <- RenumberEdges(num_to_parent(.TreeNumber.to.C(x), nTip),
+                        seq_len(nTip + nTip - 2L))
   structure(list(edge = do.call(cbind, edge),
                  tip.label = tipLabels,
                  Nnode = nTip - 1L),
@@ -205,6 +215,11 @@ as.phylo.TreeNumber <- function (x, nTip = attr(x, 'nTip'),
             class = 'phylo')
 }
 
+#' @export
+as.integer64.TreeNumber <- function (x, ...) {
+  structure(x[1], class = 'integer64')
+}
+
 #' Print `TreeNumber` object
 #'
 #' S3 method for objects of class `TreeNumber`.
@@ -215,11 +230,17 @@ as.phylo.TreeNumber <- function (x, nTip = attr(x, 'nTip'),
 #' @export
 print.TreeNumber <- function (x, ...) {
   nTip <- attr(x, 'nTip')
-  cat("Phylogenetic tree number", .PrintedTreeNumber(x), "of", NUnrooted(nTip),
-      "\n", nTip, "tips:", paste0(attr(x, 'tip.labels')))
+  cat("Phylogenetic tree number", .PrintedTreeNumber(x), "of",
+      .PrintNUnrooted(nTip), "\n",
+      nTip, "tips:", paste0(attr(x, 'tip.labels')))
+}
+
+#' @keywords internal
+.PrintNUnrooted <- function (n) {
+  if (n < 15L || n > 19L) NUnrooted(n) else paste0(NUnrooted64(n))
 }
 
 #' @keywords internal
 .PrintedTreeNumber <- function (x) {
-  paste0(c(x[1], formatC(x[-1], digits = 9L, flag='0')), collapse = '')
+  paste0(structure(x, class = 'integer64'))
 }
